#include <PalmOS.h>
#include <PalmCompatibility.h>
#include <Window.h>
#include <Bitmap.h>
#include <BmpGlue.h>
#include <SysEvtMgr.h>
#include <Day.h>
#include <SelDay.h>
#include <SelTime.h>
#include <SelTimeZone.h>
#include "Percurse.h"

// This application requires PalmOS 2.0 or later
#define ROM_VERSION_REQUIRED	0x02000000

#define	CreatorID				'PERC'
#define DBName					"PercurseDatabaseDB"
#define CATEGName				"PercCateg"
#define DBType					'DATA'

static DmOpenRef		db;
static UInt16			dbCard;
static LocalID			dbID;

static DmOpenRef		categorydb = NULL;
static LocalID			categdbID;

static Int16			nbRec = 0, curRec = 0;
static Int16			categnbRec = 0, categcurRec = 0;
static Int16 			categoryMaxId=0;


static char				buf[20];
static Int16 			day, 
						month, 
						year;
static UInt32 			currentSeconds;
static DateTimeType 	currentDate;
static UInt32			totalBytes, 
						dataBytes;

static Int16 			startHour;
static Int16 			startMinute;
static Int16 			endHour;
static Int16 			endMinute;
static Int16 			categoryId;
static Int16 			categoryIdIndex = -1;

typedef struct DBDistanceType
{
	Int16 day, month, year;
	Int16 startHour, startMinute;
	Int16 endHour, endMinute;
	Int32 distance;
	Int16 categoryId;
	char	description[21];
} DBDistanceType;
typedef DBDistanceType*	DBDistanceTypePtr;

typedef struct DBCategoryType
{
	Int16 categoryId;
	char	name[11];
	char	description[11];
	Int32 calcDist;
	Int32 calcTime;
	Int32 calcAux1;
	Int32 calcAux2;
} DBCategoryType;
typedef DBCategoryType*	DBCategoryTypePtr;


static DBDistanceType	dbRecord;
static DBCategoryType	categdbRecord;

Word 					previousFormID;

void MainFormDrawForm();

static int SeekRecord(UInt16 index) {

	MemHandle			recH;
	DBDistanceTypePtr	recP;
	UInt16				attr;
	UInt32				uniqueID;
	char				buf[30];
	
	if (recH = (MemHandle)DmQueryRecord(db, index))	{
		if (DmRecordInfo(db, index, &attr, &uniqueID, NULL)) {
			FrmCustomAlert(ERRORALERT, "Unable to get record info", "", "");
			return;
		}

		recP = (DBDistanceTypePtr)MemHandleLock(recH);

		dbRecord.day = recP->day;
		dbRecord.month=recP->month;
		dbRecord.year=recP->year;

		dbRecord.startHour=recP->startHour;
		dbRecord.startMinute=recP->startMinute;

		dbRecord.endHour=recP->endHour;
		dbRecord.endMinute=recP->endMinute;

		dbRecord.categoryId=recP->categoryId;
		
		dbRecord.distance=recP->distance;

		StrCopy(dbRecord.description, recP->description);
		
		MemHandleUnlock(recH);

		return 1;
		
	} else {

		curRec = -1;
		
		return 0;
	}
}

static Char* getMonth(UInt16 month) {

	switch (month) {
		case 1:
			return "Jan";
		case 2:
			return "Feb";
		case 3:
			return "Mar";
		case 4:
			return "Apr";
		case 5:
			return "May";
		case 6:
			return "Jun";
		case 7:
			return "Jul";
		case 8:
			return "Aug";
		case 9:
			return "Sep";
		case 10:
			return "Oct";
		case 11:
			return "Nov";
		case 12:
			return "Dec";
	}
}

static Char* GetField(UInt16 formID, UInt16 fieldID) {

	FormPtr 	frm;
	FieldPtr	fld;
	UInt16		obj;
	Char		*p;
	
	frm = FrmGetFormPtr(formID);
	obj = FrmGetObjectIndex(frm, fieldID);
	fld = (FieldPtr)FrmGetObjectPtr(frm, obj);
	p = FldGetTextPtr(fld);

	return (p ? p : "");
}


static Int16 GetControl(UInt16 formID, UInt16 controlID) {

	FormPtr 	frm;
	UInt16		obj;
	ControlPtr	ctl;

	frm = FrmGetFormPtr(formID);
	obj = FrmGetObjectIndex(frm, controlID);
	ctl = (ControlPtr)FrmGetObjectPtr(frm, obj);

	return CtlGetValue(ctl);
}

static void SetControl(UInt16 formID, UInt16 controlID, Int16 value) {

	FormPtr 	frm;
	UInt16		obj;
	ControlPtr ctl;

	frm = FrmGetFormPtr(formID);
	obj = FrmGetObjectIndex(frm, controlID);
	ctl = (ControlPtr)FrmGetObjectPtr(frm, obj);
	CtlSetValue (ctl, value);
}

static FieldPtr SetField(UInt16 formID, UInt16 fieldID, MemPtr str) {

	FormPtr 	frm;
	UInt16		obj;
	FieldPtr	fld;
	CharPtr		p;
	VoidHand	h;
	
	frm = FrmGetFormPtr(formID);
	obj = FrmGetObjectIndex(frm, fieldID);
	fld = (FieldPtr)FrmGetObjectPtr(frm, obj);
	h = (VoidHand)FldGetTextHandle(fld);

	if (h == NULL) {
		h = MemHandleNew (FldGetMaxChars(fld)+1);
		ErrFatalDisplayIf(!h, "No Memory");
	}
	
	p = (CharPtr)MemHandleLock(h);
	StrCopy(p, str);
	MemHandleUnlock(h);
	
	FldSetTextHandle(fld, (Handle)h);
}

void getTimeString(Int16 h, Int16 m, Char *c) {

	char	hourAuxBuf[20];
	char	minuteAuxBuf[20];

	if (h < 10)
		StrCopy(hourAuxBuf,"0");
	else
		StrCopy(hourAuxBuf,"");
	if (m < 10)
		StrCopy(minuteAuxBuf,"0");
	else
		StrCopy(minuteAuxBuf,"");
	StrPrintF(c, "%s%d:%s%d", hourAuxBuf, h, minuteAuxBuf, m);
}

void ResetMainForm() {

	currentSeconds = TimGetSeconds ();
	TimSecondsToDateTime (currentSeconds, &currentDate);

	month=currentDate.month;
	day = currentDate.day;
	year=currentDate.year;
	StrPrintF(buf, "%2d %s %4d", day, getMonth(month), year);
	SetField(MAIN_FORM, DATEFIELD, buf);

	categoryId = -1;
	categoryIdIndex = -1;
	
	startHour = currentDate.hour;
	endHour = startHour;
	startMinute = currentDate.minute;
	endMinute = startMinute;
	TimeToAscii(startHour, startMinute, PrefGetPreference(prefTimeFormat), buf);
	SetField(MAIN_FORM, STARTTIMEFIELD, buf);

	TimeToAscii(endHour, endMinute, PrefGetPreference(prefTimeFormat), buf);
	SetField(MAIN_FORM, ENDTIMEFIELD, buf);
}

void CategorieFormDrawForm() {
	
	FormPtr form = FrmGetActiveForm();
		
		if (categnbRec > 0)
		StrPrintF(buf, "%2d/%-2d", categcurRec+1, categnbRec);
	else 
		StrCopy(buf, "");
		
	SetField(CATEG_FORM, CATEGMAINCURSOR, buf);		


	FrmDrawForm(form);
}


void ExitItemScript(EventPtr event) { 

   EventType t = {appStopEvent, 1, 0, 0, 0, 0};
   EvtAddEventToQueue(&t);
}


static void SeekCategorieRecord(UInt16 index, int draw) {

	MemHandle			recH;
	DBCategoryTypePtr	recP;
	UInt16				attr;
	UInt32				uniqueID;
	char				buf[30];
	
	if (recH = (MemHandle)DmQueryRecord(categorydb, index)) {
		if (DmRecordInfo(categorydb, index, &attr, &uniqueID, NULL)) {
		
			FrmCustomAlert(ERRORALERT, "Unable to get record info", "", "");
			return;
		}

		recP = (DBCategoryTypePtr)MemHandleLock(recH);

		categdbRecord.categoryId = recP->categoryId;
		categoryId=categdbRecord.categoryId;

		StrCopy(categdbRecord.name, recP->name);
		StrCopy(categdbRecord.description, recP->description);

		MemHandleUnlock(recH);
		
	} else {
	
		categdbRecord.categoryId = 1;
		StrCopy(categdbRecord.name, "");
		StrCopy(categdbRecord.description, "");

		categcurRec = -1;
	}

	if (draw == 1) 
		MainFormDrawForm();
}

static void SeekCategorieByIdRecord(UInt16 id) {

	int i = 0;
	
	for (i =0; i < categnbRec ; i++) {
		SeekCategorieRecord(i, 0);
		if (categdbRecord.categoryId == id) {
			categoryIdIndex = i;
			categoryId=categdbRecord.categoryId;
			SeekCategorieRecord(i, 1);
		

			return;
		}
	}
}


static void SendToLocalMemopad() {

	int i = 0;
 	ExgSocketType exgSocket;
 	Err err = 0;

 	// message that will be sent
 	// to memopad
	Char	*fileName = "Percurse.csv\n";
	Char	*commaBuf = ",";
	Char	*quotesBuf = "\"";
	Char	*newLineBuf = "\n";
  	char	buf[40];

 	// important to init structure to zeros...
 	MemSet(&exgSocket,sizeof(exgSocket),0);
 	exgSocket.description = "Exporting Percurses to Memo";
 	// will comm. with memopad app
 	exgSocket.name = "Memo.txt";
 	// set to comm. with local machine (PIM) only
 	exgSocket.localMode = 1;

 	err = ExgPut(&exgSocket);
 	if (err == 0) {
		ExgSend(&exgSocket,fileName,StrLen(fileName),&err);

		for (i = 0; i < nbRec;i++) {
			SeekRecord(i);

			DateToAscii (dbRecord.month, dbRecord.day, dbRecord.year, dfMDYWithSlashes,buf);
			ExgSend(&exgSocket,buf,StrLen(buf),&err);

			ExgSend(&exgSocket,commaBuf,StrLen(commaBuf),&err);

			TimeToAscii(dbRecord.startHour, dbRecord.startMinute, PrefGetPreference(prefTimeFormat), buf);
			ExgSend(&exgSocket,buf,StrLen(buf),&err);

			ExgSend(&exgSocket,commaBuf,StrLen(commaBuf),&err);

			TimeToAscii(dbRecord.endHour, dbRecord.endMinute, PrefGetPreference(prefTimeFormat), buf);
			ExgSend(&exgSocket,buf,StrLen(buf),&err);
									
			ExgSend(&exgSocket,commaBuf,StrLen(commaBuf),&err);

			SeekCategorieByIdRecord(dbRecord.categoryId);
			ExgSend(&exgSocket,categdbRecord.name,StrLen(categdbRecord.name),&err);

			ExgSend(&exgSocket,commaBuf,StrLen(commaBuf),&err);

			StrIToA(buf, dbRecord.distance);
			ExgSend(&exgSocket,buf,StrLen(buf),&err);

			ExgSend(&exgSocket,newLineBuf,StrLen(newLineBuf),&err);
			
			
			
	
		}
		ExgDisconnect(&exgSocket,err);
    }
}


static int findRecWithCategId(UInt16 categIdtoFind) {

	int i = 0;
	
	for (i =0; i < nbRec ; i++) {
		SeekRecord(i);
		if (dbRecord.categoryId == categIdtoFind) {
			return 1;
		}
	}
	return 0;
}

static void SelectRecord(UInt16 index) {

	MemHandle			recH;
	DBDistanceTypePtr	recP;
	UInt16				attr;
	UInt32				uniqueID;
	char				buf[30];
	
	if (recH = (MemHandle)DmQueryRecord(db, index)) {
		if (DmRecordInfo(db, index, &attr, &uniqueID, NULL)) {
		
			FrmCustomAlert(ERRORALERT, "Unable to get record info", "", "");
			return;
		}

		recP = (DBDistanceTypePtr)MemHandleLock(recH);

		StrPrintF(buf, "%d %s %d", recP->day, getMonth(recP->month), recP->year);
		day = recP->day;
		month = recP->month;
		year=recP->year;
		SetField(MAIN_FORM, DATEFIELD, buf);
		
		TimeToAscii(recP->startHour, recP->startMinute, PrefGetPreference(prefTimeFormat), buf);		
		SetField(MAIN_FORM, STARTTIMEFIELD, buf);
		startHour = recP->startHour;
		startMinute = recP->startMinute;

		TimeToAscii(recP->endHour, recP->endMinute, PrefGetPreference(prefTimeFormat), buf);		
		SetField(MAIN_FORM, ENDTIMEFIELD, buf);
		endHour = recP->endHour;
		endMinute = recP->endMinute;

		StrIToA(buf, recP->distance);
		SetField(MAIN_FORM, DISTANCEFIELD, buf);
		
		SeekCategorieByIdRecord(recP->categoryId);
		
		SetField(MAIN_FORM, COMMENTFIELD, recP->description);
		
		MemHandleUnlock(recH);
		
		
		
	} else {
	
		ResetMainForm();

		curRec = -1;
	}

	if (nbRec > 0)
		StrPrintF(buf, "%2d/%-2d", curRec+1, nbRec);
	else 
		StrCopy(buf, "");
		
	SetField(MAIN_FORM, MAINCURSOR, buf);		

	FrmDrawForm(FrmGetFormPtr(MAIN_FORM));
}
void MainFormDrawForm() {
	
	FormPtr form = FrmGetActiveForm();
		
	StrPrintF(buf, "%2d %s %4d", day, getMonth(month), year);
	SetField(MAIN_FORM, DATEFIELD, buf);

	TimeToAscii(startHour, startMinute, PrefGetPreference(prefTimeFormat), buf);
	SetField(MAIN_FORM, STARTTIMEFIELD, buf);

	TimeToAscii(endHour, endMinute, PrefGetPreference(prefTimeFormat), buf);
	SetField(MAIN_FORM, ENDTIMEFIELD, buf);
	
	if (categoryIdIndex == -1)
		StrCopy(buf, "None");
	else {
		StrCopy(buf, categdbRecord.name);
	}
	SetField(MAIN_FORM, CATEGORYFIELD, buf);

	FrmDrawForm(form);
}

static Int16 GetMaxCategorieId() {

	MemHandle			recH;
	DBCategoryTypePtr	recP;
	UInt16				attr;
	UInt32				uniqueID;
	char				buf[30];
	Int16				maxId = 0;
	
	
	if (recH = (MemHandle)DmQueryRecord(categorydb, categnbRec-1)) {
		if (DmRecordInfo(categorydb, categnbRec-1, &attr, &uniqueID, NULL)) {
		
			FrmCustomAlert(ERRORALERT, "Unable to get record info", "", "");
			return;
		}

		recP = (DBCategoryTypePtr)MemHandleLock(recH);

		maxId = recP->categoryId;

		MemHandleUnlock(recH);
		
	} else {
	
		maxId = 1;
	}

	return maxId;
}


static void SelectCategorieRecord(UInt16 index) {

	MemHandle			recH;
	DBCategoryTypePtr	recP;
	UInt16				attr;
	UInt32				uniqueID;
	char				buf[30];
	
	if (recH = (MemHandle)DmQueryRecord(categorydb, index)) {
		if (DmRecordInfo(categorydb, index, &attr, &uniqueID, NULL)) {
		
			FrmCustomAlert(ERRORALERT, "Unable to get record info", "", "");
			return;
		}

		recP = (DBCategoryTypePtr)MemHandleLock(recH);

		StrCopy(buf, recP->name);
		SetField(CATEG_FORM, CATEGNAMEFIELD, buf);
		StrCopy(buf, recP->description);
		SetField(CATEG_FORM, CATEGCOMMENTFIELD, buf);
		categoryId = recP->categoryId;
		MemHandleUnlock(recH);
		
	} else {
	
		StrCopy(buf, "Default");
		SetField(CATEG_FORM, CATEGNAMEFIELD, buf);
		StrCopy(buf, "None");
		SetField(CATEG_FORM, CATEGCOMMENTFIELD, buf);

		categcurRec = -1;
	}

	if (categnbRec > 0)
		StrPrintF(buf, "%2d/%-2d", categcurRec+1, categnbRec);
	else 
		StrCopy(buf, "");
		
	SetField(CATEG_FORM, CATEGMAINCURSOR, buf);		

	FrmDrawForm(FrmGetFormPtr(CATEG_FORM));
}


static void InsertRecord() {

	UInt16		index;
	MemHandle 	recH;
	MemPtr		recP;

	// Add a new record at the end of the database.
	index = DmNumRecords(db);
	if (!(recH = DmNewRecord(db, &index, sizeof(DBDistanceType)))) {
		FrmCustomAlert(ERRORALERT, "Unable to create record", "", "");
		return;
	}

	dbRecord.day = day;
	dbRecord.month = month;
	dbRecord.year = year;
	dbRecord.startHour = startHour;
	dbRecord.startMinute = startMinute;
	dbRecord.endHour = endHour;
	dbRecord.endMinute = endMinute;
	dbRecord.categoryId=categoryId;
	dbRecord.distance = StrAToI(GetField(MAIN_FORM, DISTANCEFIELD));

	StrCopy(dbRecord.description, GetField(MAIN_FORM, COMMENTFIELD));
	
	recP = MemHandleLock(recH);
	DmWrite(recP, 0, &dbRecord, sizeof(DBDistanceType)); 
	MemPtrUnlock(recP);

	DmReleaseRecord(db, index, true);
	
	curRec = index;
	nbRec++;
}

static void InsertCategorieRecord() {

	UInt16		index;
	MemHandle 	recH;
	MemPtr		recP;
	UInt16		maxId;
	
	maxId = GetMaxCategorieId();
	maxId++;
	
	// Add a new record at the end of the database.
	index = DmNumRecords(categorydb);
	if (!(recH = DmNewRecord(categorydb, &index, sizeof(DBCategoryType)))) {
		FrmCustomAlert(ERRORALERT, "Unable to create record", "", "");
		return;
	}

	categdbRecord.categoryId = maxId;
	StrCopy(categdbRecord.name, GetField(CATEG_FORM, CATEGNAMEFIELD));
	StrCopy(categdbRecord.description, GetField(CATEG_FORM, CATEGCOMMENTFIELD));
	
	recP = MemHandleLock(recH);
	DmWrite(recP, 0, &categdbRecord, sizeof(DBCategoryType)); 
	MemPtrUnlock(recP);

	DmReleaseRecord(categorydb, index, true);
	
	categcurRec = index;
	categnbRec++;
}

static void DeleteRecord() {

	UInt16		attr;
	Err			error;
	
	if (DmRecordInfo(db, curRec, &attr, NULL, NULL)) {
		FrmCustomAlert(ERRORALERT, "Unable to get record info", "", "");
		return;
	}
	
	if ((attr & dmRecAttrDelete) == 0)
		// delete OK: we need to find a valid "current" rec, forwards OR backwards
		if ((error = DmRemoveRecord(db, curRec)) == 0) {
		
			UInt16	seek = curRec;

			// fast forward
			while (seek < nbRec)
				if (DmQueryRecord(db, seek) == NULL)
					seek++;
				else 
					break; // found one!
						
				// found none
				if (seek >= nbRec) {
			
					seek = curRec;
					while (--seek > 0) // rewind
						if (DmQueryRecord(db, seek) != NULL) 
							break; // found one!
			}
				
			curRec = seek; // got the new "current"
			nbRec--; // decrement rec count
			SelectRecord(curRec);
		} else 
			ErrAlert(error);
	else 
		FrmCustomAlert(ERRORALERT, "Record already deleted", "", "");
}

static void DeleteCategoryRecord() {

	UInt16		attr;
	Err			error;
	
	
	if (DmRecordInfo(categorydb, categcurRec, &attr, NULL, NULL)) {
		FrmCustomAlert(ERRORALERT, "Unable to get record info", "", "");
		return;
	}
	
	if ((attr & dmRecAttrDelete) == 0)
		// delete OK: we need to find a valid "current" rec, forwards OR backwards
		if ((error = DmRemoveRecord(categorydb, categcurRec)) == 0) {
		
			UInt16	seek = categcurRec;

			// fast forward
			while (seek < categnbRec)
				if (DmQueryRecord(categorydb, seek) == NULL)
					seek++;
				else 
					break; // found one!
						
				// found none
				if (seek >= categcurRec) {
			
					seek = categcurRec;
					while (--seek > 0) // rewind
						if (DmQueryRecord(categorydb, seek) != NULL) 
							break; // found one!
			}
				
			categcurRec = seek; // got the new "current"
			categnbRec--; // decrement rec count
			SelectCategorieRecord(categcurRec);
		} else 
			ErrAlert(error);
	else 
		FrmCustomAlert(ERRORALERT, "Record already deleted", "", "");
}

static void UpdateRecord() {

	UInt16				index = curRec;
	MemHandle			recH;
	MemPtr				recP;
	UInt16				attr;
	char				buf[20];
	
	if (DmRecordInfo(db, index, &attr, NULL, NULL)) {
		FrmCustomAlert(ERRORALERT, "Unable to get record info", "", "");
		return;
	}
	
	if ((attr & dmRecAttrBusy) != 0) {
		FrmCustomAlert(ERRORALERT, "Record is locked", "", "");
		return;
	}

	if (recH = (MemHandle)DmGetRecord(db, index)) {
		if (DmSetRecordInfo(db, index, &attr, NULL)) {
			FrmCustomAlert(ERRORALERT, "Error setting record info", "", "");
			return;
		}

		dbRecord.day = day;
		dbRecord.month= month;
		dbRecord.year=year;
		dbRecord.startHour=startHour;
		dbRecord.startMinute=startMinute;
		dbRecord.endHour=endHour;
		dbRecord.endMinute=endMinute;
		dbRecord.distance = StrAToI(GetField(MAIN_FORM, DISTANCEFIELD));

		StrCopy(dbRecord.description, GetField(MAIN_FORM, COMMENTFIELD));
		
		recP = MemHandleLock(recH);
		DmWrite(recP, 0, &dbRecord, sizeof(DBDistanceType)); 
		MemPtrUnlock(recP);
	
		DmReleaseRecord(db, index, true);
	} else 
		FrmCustomAlert(ERRORALERT, "Unable to get record", "", "");
}

static void UpdateCategoryRecord() {

	UInt16				index = categcurRec;
	MemHandle			recH;
	MemPtr				recP;
	UInt16				attr;
	char				buf[20];
	
	if (DmRecordInfo(categorydb, index, &attr, NULL, NULL)) {
		FrmCustomAlert(ERRORALERT, "Unable to get record info", "", "");
		return;
	}
	
	if ((attr & dmRecAttrBusy) != 0) {
		FrmCustomAlert(ERRORALERT, "Record is locked", "", "");
		return;
	}

	if (recH = (MemHandle)DmGetRecord(categorydb, index)) {


		if (DmSetRecordInfo(categorydb, index, &attr, NULL)) {
			FrmCustomAlert(ERRORALERT, "Error setting record info", "", "");
			return;
		}

		categdbRecord.categoryId = categoryId;
		StrCopy(categdbRecord.name, GetField(CATEG_FORM, CATEGNAMEFIELD));
		StrCopy(categdbRecord.description, GetField(CATEG_FORM, CATEGCOMMENTFIELD));
		

		recP = MemHandleLock(recH);
		DmWrite(recP, 0, &categdbRecord, sizeof(DBCategoryType)); 
		MemPtrUnlock(recP);
	
		DmReleaseRecord(categorydb, index, true);

		} else 
			FrmCustomAlert(ERRORALERT, "Unable to get record", "", "");
}


Boolean MainFormHandleEvent(EventPtr event) {

   Boolean handled = false;
   	char				buf[20];

   Boolean selected;
   
   switch (event->eType) {

	  case frmLoadEvent:
		FrmSetActiveForm(FrmInitForm(event->data.frmLoad.formID));
		break;

      case frmOpenEvent:

		if (event->data.frmLoad.formID == MAIN_FORM) {
			SelectRecord(curRec = nbRec-1);
			MainFormDrawForm();
			handled = true;
		} else 
			if (event->data.frmLoad.formID == CATEG_FORM) {
				SelectCategorieRecord(categcurRec = categnbRec-1);
				handled = true;
			}

       
        break;

      case menuEvent: {
	  
			switch (event->data.ctlEnter.controlID) {

				case EXITITEM:
					ExitItemScript(event);
					break;
			   
				case EXPORTITEM:
					SendToLocalMemopad();
					handled = true;
					break;

				case CATEGORIEITEM:
					FrmGotoForm(CATEG_FORM);
					handled = true;
					break;

				case CATEGORIEBACKITEM:
					FrmGotoForm(MAIN_FORM);
					handled = true;
					break;
					
				case MAINOPTIONSHELPCMD:
					(void)FrmAlert(HELPALERT);
					handled = true;
					break;
				    
				case MAINOPTIONSABOUTCMD:
					(void)FrmAlert(ABOUTALERT);
					handled = true;
					break;
			}
		}
        handled = true;
        break;

		case ctlSelectEvent: {

			switch (event->data.ctlSelect.controlID) {
			
				case GETDATE:
				
					selected = SelectDay (selectDayByDay, &month, &day, &year, "Enter Date");
					if (selected) {
						StrPrintF(buf, "%d %s %d", day, getMonth(month), year);
						SetField(MAIN_FORM, DATEFIELD, buf);
					}
					handled = true;
					FrmDrawForm(FrmGetFormPtr(MAIN_FORM));
					break;
				
				case GETSTARTTIME:
					selected = SelectOneTime (&startHour, &startMinute, "Enter Start Time");
					if (selected) {

						TimeToAscii(startHour, startMinute, PrefGetPreference(prefTimeFormat), buf);
						SetField(MAIN_FORM, STARTTIMEFIELD, buf);
					}
					handled = true;
					FrmDrawForm(FrmGetFormPtr(MAIN_FORM));
					break;

				case GETENDTIME:
					selected = SelectOneTime (&endHour, &endMinute, "Enter End Time");
					if (selected) {
						TimeToAscii(endHour, endMinute, PrefGetPreference(prefTimeFormat), buf);
						SetField(MAIN_FORM, ENDTIMEFIELD, buf);
					}
					handled = true;
					FrmDrawForm(FrmGetFormPtr(MAIN_FORM));
					break;
					
				case SETDATETONOW:
					currentSeconds = TimGetSeconds ();
					TimSecondsToDateTime (currentSeconds, &currentDate);

					month=currentDate.month;
					day = currentDate.day;
					year=currentDate.year;
					handled = true;
					MainFormDrawForm();
					break;
					
				case SETSTARTDATETONOW:
					currentSeconds = TimGetSeconds ();
					TimSecondsToDateTime (currentSeconds, &currentDate);
					
					startHour = currentDate.hour;
					startMinute = currentDate.minute;
					handled = true;
					MainFormDrawForm();
	
					break;
					
				case SETENDDATETONOW:
					currentSeconds = TimGetSeconds ();
					TimSecondsToDateTime (currentSeconds, &currentDate);
					
					endHour = currentDate.hour;
					endMinute = currentDate.minute;
					handled = true;
					MainFormDrawForm();
				
					break;
					
				case MAININSERT:
				
					if (SeekRecord(curRec = nbRec-1)) {
						
						if (dbRecord.year > year) {
							FrmCustomAlert(ERRORALERT, "Date should be later than last entrance", "", ""); 
							break;
						} else
							if (dbRecord.year == year && dbRecord.month > month) {
								FrmCustomAlert(ERRORALERT, "Date should be later than last entrance", "", ""); 
								break;
							} else 
								if (dbRecord.year == year && dbRecord.month == month && dbRecord.day > day) {
									FrmCustomAlert(ERRORALERT, "Date should be later than last entrance", "", ""); 
									break;
								}

						if (dbRecord.distance >= StrAToI(GetField(MAIN_FORM, DISTANCEFIELD))) {
							FrmCustomAlert(ERRORALERT, "Odometer should be greater than last entrance", "", ""); 
							break;
						}
					}

					if (endHour < startHour) {
						FrmCustomAlert(ERRORALERT, "End Date should be later than Start Date", "", ""); 
						break;
					} else 
						if (endHour == startHour && endMinute <= startMinute) {
							FrmCustomAlert(ERRORALERT, "End Date should be later than Start Date", "", ""); 
							break;
						}
					if (categoryIdIndex == -1) {
						FrmCustomAlert(ERRORALERT, "No category selected. Create one and select it.", "", ""); 	categoryIdIndex = 0;
						break;
					}
					InsertRecord();
					SelectRecord(curRec);

				    handled = true;
				    break;
					
				case CATEGMAININSERT:
				
					InsertCategorieRecord();
					SelectCategorieRecord(categcurRec);
				    handled = true;
				    break;

				case MAINFIRSTREC:
					SelectRecord(curRec = 0);
				    handled = true;
				    break;

				case CATEGMAINFIRSTREC:
					SelectCategorieRecord(categcurRec = 0);
				    handled = true;
				    break;

				case MAINLASTREC:
					SelectRecord(curRec = nbRec-1);
				    handled = true;
				    break;

				case CATEGMAINLASTREC:
					SelectCategorieRecord(categcurRec = categnbRec-1);
				    handled = true;
				    break;
					
				case MAINPREVREC:
					if (curRec > 0)
						SelectRecord(--curRec);
					handled = true;
				    break;
					
				case CATEGMAINPREVREC:
					if (categcurRec > 0)
						SelectCategorieRecord(--categcurRec);
					handled = true;
				    break;

				case MAINNEXTREC:
					if (curRec < nbRec-1)
						SelectRecord(++curRec);
					handled = true;
				    break;
					
				case CATEGMAINNEXTREC:
					if (categcurRec < categnbRec-1)
						SelectCategorieRecord(++categcurRec);
					handled = true;
				    break;

				case CATSELECTMAINNEXTREC:
					if (categnbRec > 0) {
						if (categoryIdIndex == -1) {
							categoryIdIndex = 0;
							SeekCategorieRecord(categoryIdIndex = 0, 1);
						} else {
							if (categoryIdIndex < categnbRec - 1) {
								categoryIdIndex++;
								SeekCategorieRecord(categoryIdIndex, 1);
							}
						}
					}
							
					handled = true;
				    break;
					
				case CATSELECTMAINPREVREC:
					if (categnbRec - 1 > 0) {
						if (categoryIdIndex > 0) {
							categoryIdIndex--;
							SeekCategorieRecord(categoryIdIndex, 1);
						} 
					}
							
					handled = true;
				    break;
					
				case MAINDELETE:
					if (curRec >= 0)
						DeleteRecord();
				    handled = true;
				    break;
					
				case CATEGMAINDELETE:
					if (categcurRec >= 0) {
						SeekCategorieRecord(categcurRec, 0);
						if (findRecWithCategId(categdbRecord.categoryId) == 1) {
							FrmCustomAlert(ERRORALERT, "This category is being used", "", "");
							return;
						}
						DeleteCategoryRecord();
					}
				    handled = true;
				    break;

				case MAINUPDATE:
					if (curRec >= 0)
						UpdateRecord();
				    handled = true;
				    break;

				case CATEGMAINUPDATE:
					if (categcurRec >= 0)
						UpdateCategoryRecord();
				    handled = true;
				    break;

			}
			break;
		}

		case frmCloseEvent:

			handled = false;
			break;

	}

	return(handled);
}

Boolean ApplicationHandleEvent(EventPtr event) {

   FormPtr frm;
   Int formId;
   Boolean handled = false;

   if (event->eType == frmLoadEvent) {

      formId = event->data.frmLoad.formID;

      frm = FrmInitForm(formId);
      FrmSetActiveForm(frm);

	  FrmSetEventHandler(frm, MainFormHandleEvent);

      handled = true;
   }

   return handled;
}

Err StartApplication(void) {

	UInt16	attributes;
	UInt32	locks;
	Err		retcode = 0;

	// Find the application's data file.  If it doesn't exist create it.
	if (db = DmOpenDatabaseByTypeCreator(DBType, CreatorID, dmModeReadWrite | dmModeShowSecret))
		DmOpenDatabaseInfo(db, &dbID, NULL, NULL, &dbCard, NULL);
	else {
		if (retcode = DmCreateDatabase(0, DBName, CreatorID, DBType, false))
			return retcode;
			
		if (!(db = DmOpenDatabaseByTypeCreator(DBType, CreatorID, dmModeReadWrite | dmModeShowSecret)))
			return dmErrCantOpen;
	
		// Set the backup bit.  This is to aid syncs with non Palm software.
		DmOpenDatabaseInfo(db, &dbID, NULL, NULL, &dbCard, NULL);
		DmDatabaseInfo(dbCard, dbID, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
		attributes |= dmHdrAttrBackup;
		DmSetDatabaseInfo(dbCard, dbID, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
	}

    DmGetDatabaseLockState(db, NULL, NULL, &locks);
    DmDatabaseProtect(dbCard, dbID, (locks > 0) ? false : true);			// unprotect/protect DB accordingly, to balance AppClose similar call

	nbRec = DmNumRecords(db);

	
	categdbID = DmFindDatabase(0, CATEGName);

	if (categdbID > 0) 
		categorydb = DmOpenDatabase(0, categdbID, dmModeReadWrite);

	if (!categorydb) {

		retcode = DmCreateDatabase(0, CATEGName, CreatorID, DBType, false);

		if (retcode == 0)	

			categorydb = DmOpenDatabase(0, DmFindDatabase(0, CATEGName), dmModeReadWrite);
			
		else
		
			return retcode;
			
		// Set the backup bit.  This is to aid syncs with non Palm software.
		DmOpenDatabaseInfo(categorydb, &categdbID, NULL, NULL, &dbCard, NULL);
		DmDatabaseInfo(dbCard, categdbID, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
		attributes |= dmHdrAttrBackup;
		DmSetDatabaseInfo(dbCard, categdbID, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
			
	}
	
    DmGetDatabaseLockState(categorydb, NULL, NULL, &locks);
    DmDatabaseProtect(dbCard, categdbID, (locks > 0) ? false : true);			// unprotect/protect DB accordingly, to balance AppClose similar call

	categnbRec = DmNumRecords(categorydb);

	FrmGotoForm(MAIN_FORM);
	return retcode;
}

static Err StopApplication() {

	UInt32	locks;
	Boolean	protect = false;
	Err		retcode;
	
    FrmCloseAllForms();
    
    DmGetDatabaseLockState(db, NULL, NULL, &locks);
	// any record locked?
    if (protect = (locks > 0)) {
	
    	char	buf[40];
    	
    	StrPrintF(buf, "%ld record(s) still locked.", locks);
    	FrmCustomAlert(ERRORALERT, buf, "Database will be closed protected.", "If you don't unlock every record, you WON'T be allowed to delete the database.");
    }
    DmDatabaseProtect(dbCard, dbID, protect);				// protect/unprotect DB accordingly
	DmCloseDatabase(db);

    DmGetDatabaseLockState(categorydb, NULL, NULL, &locks);
	// any record locked?
    if (protect = (locks > 0)) {
	
    	char	buf[40];
    	
    	StrPrintF(buf, "%ld record(s) still locked.", locks);
    	FrmCustomAlert(ERRORALERT, buf, "Database will be closed protected.", "If you don't unlock every record, you WON'T be allowed to delete the database.");
    }
    DmDatabaseProtect(dbCard, categdbID, protect);				// protect/unprotect DB accordingly
	DmCloseDatabase(categorydb);

    return retcode;
}

void EventLoop(void) {

   EventType event;
   Word error;

   do {

      EvtGetEvent(&event, evtWaitForever);

      if (! SysHandleEvent(&event))
         if (! MenuHandleEvent(0, &event, &error))
            if (! ApplicationHandleEvent(&event))
               FrmDispatchEvent(&event);
			   
   } while (event.eType != appStopEvent);
}

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags) {

   Err err = 0;
   UInt32 romVersion;

   // Minimal ROM version requirement
   FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);	
   if (romVersion < ROM_VERSION_REQUIRED) {
		FrmAlert(ROMINCOMPATIBLEALERT);
		return(sysErrRomIncompatible);
   }

   if (cmd == sysAppLaunchCmdNormalLaunch) {

      if ((err = StartApplication()) == 0) {

         EventLoop();

		 StopApplication();
	     
      }
   }

   return err;
}
